@@ -1,5 +1,5 @@
 #! /bin/bash
-pkg=`grep distroverpkg /etc/yum.conf |awk -F= '{print $2}'`
+pkg="redhat-release"
 releasever=`rpm -q --qf "%{version}" $pkg`
 base=`uname -m`
 echo "ftp://ftp.redhat.com/pub/redhat/linux/\
