@@ -12,25 +12,32 @@
 #include <stdlib.h>
 #include <string.h>
 #include <error.h>
+#if ENABLE_NLS
 #include <locale.h>
+#endif
 #include <argp.h>
 #include <elfutils/libdwfl.h>
-#ifdef HAVE_ELFUTILS_VERSION_H
 #include <elfutils/version.h>
-#endif
 #include <dwarf.h>
 #include <obstack.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include "loc2c.h"
 
-#if !defined(_ELFUTILS_PREREQ)
-// make a dummy PREREQ check for elfutils < 0.138
-#define _ELFUTILS_PREREQ(major, minor) (0 >= 1)
-#endif
-
 #define _(msg) msg
 
+/* Using this structure in the callback (to handle_function) allows us
+   to print out the function's address */
+struct dwfunc
+{
+  Dwfl *dwfl;
+  Dwarf_Die *cu;
+  Dwarf_Addr dwbias;
+  char * funcname;
+  uintmax_t pc;
+  bool matchdebug;
+};
+
 static void __attribute__ ((noreturn))
 fail (void *arg __attribute__ ((unused)), const char *fmt, ...)
 {
@@ -177,22 +184,10 @@ handle_fields (struct obstack *pool,
 	    }
 	  else
 	    {
-	      /* We expect a block or a constant.  In older elfutils,
-		 dwarf_getlocation_addr would not handle the constant for
-		 us, but newer ones do.  For older ones, we work around
-		 it by faking an expression, which is what newer ones do.  */
-	      size_t locexpr_len;
+	      /* We expect a block or a constant.  */
+	      size_t locexpr_len = 0;
 	      const Dwarf_Op *locexpr;
-#if !_ELFUTILS_PREREQ (0,142)
-	      Dwarf_Op offset_loc = { .atom = DW_OP_plus_uconst };
-	      if (dwarf_formudata (&attr_mem, &offset_loc.number) == 0)
-		{
-		  locexpr = &offset_loc;
-		  locexpr_len = 1;
-		}
-	      else
-#endif
-		locexpr = get_location (cubias, pc, &attr_mem, &locexpr_len);
+	      locexpr = get_location (cubias, pc, &attr_mem, &locexpr_len);
 	      c_translate_location (pool, NULL, NULL, NULL,
 				    1, cubias, pc, &attr_mem,
 				    locexpr, locexpr_len,
@@ -355,7 +350,7 @@ handle_variable (Dwarf_Die *lscopes, int lnscopes, int out,
 	error (2, 0, _("cannot get location of variable: %s"),
 	       dwarf_errmsg (-1));
 
-      size_t locexpr_len;
+      size_t locexpr_len = 0;
       const Dwarf_Op *locexpr = get_location (cubias, pc,
 					      &attr_mem, &locexpr_len);
 
@@ -459,6 +454,73 @@ print_vars (unsigned int indent, Dwarf_Die *die)
     while (dwarf_siblingof (&child, &child) == 0);
 }
 
+static int
+handle_function (Dwarf_Die *funcdie, void *arg)
+{
+  struct dwfunc *a = arg;
+  Dwarf_Addr entrypc;
+  int result;
+  int i = 0;
+  Dwarf_Addr *bkpts = NULL;
+  if (arg == NULL)
+    error (2, 0, "Error, dwfl structure empty");
+  const char *name = dwarf_diename (funcdie);
+  if (name == NULL)
+    error (2, 0, "Error, dwarf_diename returned NULL from Dwarf_die");
+  if (dwarf_entrypc (funcdie, &entrypc) == 0)
+    {
+      entrypc += a->dwbias;
+      result = dwarf_entry_breakpoints (funcdie, &bkpts);
+      if (a->funcname == NULL)
+	{
+	  printf ("%-35s  %#.16" PRIx64, name, entrypc);
+	  /* error check the result is greater than 0 */
+	  if (result > 0)
+	    {
+	      /* the formatting changes as a looka-head if we
+		 need different ending whitespace chars the
+		 location is a combination of the address +
+		 the offset (bias) */
+	      for (i = 0; i < result; i++)
+		printf (" %#.16" PRIx64 "%s", bkpts[i] + a->dwbias,
+			i == result - 1 ? "\n" : "");
+	      free (bkpts);
+	    }
+	}
+      else if (a->funcname != NULL && (!strcmp (a->funcname, name)))
+	{
+	  if (!a->matchdebug)
+	    {
+	      entrypc += a->dwbias;
+	      a->pc = entrypc;
+	      free (bkpts);
+	      return DWARF_CB_ABORT;
+	    }
+	  else if (result == 1 && a->matchdebug)
+	    {
+	      a->pc = (bkpts[0] + a->dwbias);
+	      free (bkpts);
+	      return DWARF_CB_ABORT;
+	    }
+	  else if (result > 1 && a->matchdebug)
+	    {
+	      printf ("Function: %-35s has multiple breakpoints: ", name);
+	      for (i = 0; i < result; i++)
+		printf (" %#.16" PRIx64 "%s", bkpts[i] + a->dwbias,
+			i == result - 1 ? "\n" : "");
+	      free (bkpts);
+	      return DWARF_CB_ABORT;
+	    }
+	  else
+	    {
+	      error (2, 0, "Error, dwarf_entry_breakpoints returned an error( %s )\n",
+		     dwarf_errmsg (result));
+	    }
+	}
+    }
+  return 0;
+}
+
 #define INDENT 4
 
 int
@@ -467,8 +529,10 @@ main (int argc, char **argv)
   /* We use no threads here which can interfere with handling a stream.  */
   (void) __fsetlocking (stdout, FSETLOCKING_BYCALLER);
 
+#if ENABLE_NLS
   /* Set locale.  */
   (void) setlocale (LC_ALL, "");
+#endif
 
   const struct argp_child argp_children[] =
     {
@@ -479,32 +543,59 @@ main (int argc, char **argv)
     {
       .children = argp_children,
       .args_doc = "ADDRESS\n\
+FUNCTIONNAME\n\
 ADDRESS VARIABLE [FIELD...]\n\
+FUNCTIONNAME VARIABLE [FIELD...]\n\
 ADDRESS VARIABLE [FIELD...] =",
       .doc = "Process DWARF locations visible with PC at ADDRESS.\v\
 ADDRESS must be in %i format (i.e. \"0x...\").\n\
-In the first form, display the scope entries containing ADDRESS,\
- and the variables/parameters visible there, with their type names. \
- Hex numbers within \"[...]\" are DIE offsets.\n\
-In the second form, emit a C code fragment to access a variable. \
- Each FIELD argument is the name of a member of an aggregate type\
- (or pointer to one);\
+FUNCTIONNAME must be exact or prepended with '@' character to specify \
+ breakpoint entry.\n\
+In the first and second form, display the scope entries containing \
+ ADDRESS or FUNCTIONNAME, and the variables/parameters visible there, \
+ with their type names. Hex numbers within \"[...]\" are DIE offsets.\n\
+In the third and fourth form, emit a C code fragment to access a \
+ variable. Each FIELD argument is the name of a member of an aggregate \
+ type (or pointer to one);\
  \"+INDEX\" to index into an array type (or offset a pointer type);\
  or \"\" (an empty argument) to dereference a pointer type.\n\
-In the third form, the access is a store rather than a fetch."
+In the fifth form, the access is a store rather than a fetch."
+
     };
   Dwfl *dwfl = NULL;
   int argi;
   (void) argp_parse (&argp, argc, argv, 0, &argi, &dwfl);
   assert (dwfl != NULL);
-
+  struct dwfunc a = { .dwfl = dwfl, .funcname = NULL,
+		      .pc = 0, .matchdebug = 0 };
   if (argi == argc)
-    error (2, 0, "need address argument");
+    {
+      printf ("%-35s  %-18s %s\n",
+	      "Function", "Address", "Debug Entry Address(s)");
+      while ((a.cu = dwfl_nextcu(a.dwfl, a.cu, &a.dwbias)) != NULL)
+	dwarf_getfuncs (a.cu, &handle_function, &a.dwfl, 0);
+      return 0;
+    }
 
   char *endp;
   uintmax_t pc = strtoumax (argv[argi], &endp, 0);
   if (endp == argv[argi])
-    error (2, 0, "bad address argument");
+    {
+      a.funcname = argv[argi];
+      /* We need to check and adjust for a possible '@' character */
+      if (a.funcname[0] == '@')
+	{
+	  a.funcname++;
+	  a.matchdebug = 1;
+	}
+      
+      while ((a.cu = dwfl_nextcu(a.dwfl, a.cu, &a.dwbias)) != NULL)
+	dwarf_getfuncs (a.cu, &handle_function, &a.dwfl, 0);
+      /* If a.pc isn't set, that means the function specified wasn't found */
+      if (!a.pc)
+	error (EXIT_FAILURE, 0, "function: '%s' not found\n", argv[argi]);
+      pc = a.pc;
+    }
 
   Dwarf_Addr cubias;
   Dwarf_Die *cudie = dwfl_addrdie (dwfl, pc, &cubias);
@@ -609,7 +700,6 @@ In the third form, the access is a store rather than a fetch."
 	{
 	  Dwarf_Op *cfa_ops = NULL;
 
-#if _ELFUTILS_PREREQ(0,142)
 	  size_t cfa_nops;
 	  Dwarf_Addr bias;
 	  Dwfl_Module *module = dwfl_addrmodule (dwfl, pc);
@@ -634,7 +724,6 @@ In the third form, the access is a store rather than a fetch."
 		    }
 		}
 	    }
-#endif
 
 	  handle_variable (scopes, n, out, cubias, &vardie, pc, cfa_ops,
 			   &argv[argi]);
