@@ -1280,6 +1280,8 @@ void sign_file (
 
   for (;;)
     {
+      // No need for PR_Read_Complete here, since we're already managing multiple
+      // reads to a fixed size buffer.
       numBytes = PR_Read (local_file_fd, buffer, sizeof (buffer));
       if (numBytes == 0)
 	break;	/* EOF */
@@ -1339,4 +1341,28 @@ void sign_file (
     PR_Close (local_file_fd);
 }
 
+// PR_Read() is not guaranteed to read all of the requested data in one call.
+// Iterate until all of the requested data has been read.
+// Return the same values as PR_Read() would.
+PRInt32 PR_Read_Complete (PRFileDesc *fd, void *buf, PRInt32 requestedBytes)
+{
+  // Read until EOF or until the expected number of bytes has been read.
+  // PR_Read wants (void*), but we need (char *) to do address arithmetic.
+  char *buffer = (char *)buf;
+  PRInt32 totalBytes;
+  PRInt32 bytesRead;
+  for (totalBytes = 0; totalBytes < requestedBytes; totalBytes += bytesRead)
+    {
+      // Now read the data.
+      bytesRead = PR_Read (fd, (void *)(buffer + totalBytes), requestedBytes - totalBytes);
+      if (bytesRead == 0)
+	break;	// EOF
+      if (bytesRead < 0)
+	return bytesRead; // Error
+    }
+
+  // Return the number of bytes we managed to read.
+  return totalBytes;
+}
+
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
