@@ -19,28 +19,6 @@
 #define WARN_STRING "WARNING: "
 #define ERR_STRING "ERROR: "
 
-// XXX for now, all IO is going in-process to stdout/stderr via
-// _stp_out/_stp_err; see runtime/dyninst/runtime.h for initialization.
-
-static FILE* _stp_out = NULL;
-static FILE* _stp_err = NULL;
-
-
-/* Clone a FILE* for private use.  On error, fallback to the original. */
-static FILE* _stp_clone_file(FILE* file)
-{
-    int fd = dup(fileno(file));
-    if (fd != -1) {
-	fcntl(fd, F_SETFD, FD_CLOEXEC);
-	FILE* newfile = fdopen(fd, "wb");
-	if (newfile)
-	    return newfile;
-
-	close(fd);
-    }
-    return file;
-}
-
 enum code { INFO=0, WARN, ERROR, DBUG };
 
 static void _stp_vlog (enum code type, const char *func, int line,
@@ -52,13 +30,22 @@ static void _stp_vlog (enum code type, const char *func, int line,
 {
 	size_t start = 0;
 	size_t num;
-	char tmp[STP_LOG_BUF_LEN];
-	char *buf = _stp_dyninst_transport_log_buffer();
+	char local_buffer[STP_LOG_BUF_LEN];
+	char *buf;
 
-	/* If we can't get a buffer, the transport must be not up yet.
-	 * Instead use our temporary buffer. */
-	if (buf == NULL)
-		buf = tmp;
+	/* If we're writing a debug message, just use the temporary
+	 * buffer. These messages end up going through the normal
+	 * transport path, not the OOB data path. */
+	if (type == DBUG) {
+		buf = local_buffer;
+	}
+	else {
+		/* If we can't get a buffer, the transport must be not
+		 * up yet.  Instead use our temporary buffer. */
+		buf = _stp_dyninst_transport_log_buffer();
+		if (buf == NULL)
+			buf = local_buffer;
+	}
 
 	if (type == DBUG) {
 		start = snprintf(buf, STP_LOG_BUF_LEN, "%s:%d: ", func, line);
@@ -93,19 +80,20 @@ static void _stp_vlog (enum code type, const char *func, int line,
 		buf[num + start] = '\0';
 	}
 
-	/* If we're using the temporary buffer, just output it to
-	 * _stp_err. */
-	if (buf == tmp) {
-		fprintf(_stp_err, "%s", buf);
-	}
-	else if (type != DBUG) {
-                /* NB: don't explicitly send the \0 terminator. */
+	if (buf != local_buffer) {
+		/* If we successfully got log space, send it as OOB now.
+		 * NB: don't explicitly send the \0 terminator. */
 		_stp_dyninst_transport_write_oob_data(buf, num + start);
 	}
-	else {
+	else if (type == DBUG) {
+		/* Debug messages go over the normal data path.  */
 		_stp_print(buf);
 		_stp_print_flush();
 	}
+	else {
+		/* Fallback to local stderr as a last resort.  */
+		fprintf(stderr, "%s", buf);
+	}
 }
 
 /** Prints warning.
@@ -171,6 +159,9 @@ static void _stp_dbug (const char *func, int line, const char *fmt, ...)
 	va_end(args);
 }
 
-#define _stp_exit() do { } while (0) // no transport, no action yet
+static void _stp_exit (void)
+{
+	_stp_dyninst_transport_request_exit();
+}
 
 #endif /* _STAPDYN_IO_C_ */
