@@ -220,12 +220,10 @@ static void _stp_shm_finalize(void)
 // Tear down everything we created... *sniff*
 // NB: Make sure not to reference any memory within after this!
 // (Other processes may still have their own mmap reference though.)
+// Don't destroy things in shm that may still be used by other processes!
 static void _stp_shm_destroy(void)
 {
 	if (_stp_shm_base) {
-		// Tear down the contexts themselves.
-		__stp_runtime_contexts_free();
-
 		munmap(_stp_shm_base, _stp_shm_size);
 		_stp_shm_base = NULL;
 		_stp_shm_size = 0;
