@@ -16,11 +16,7 @@
  * @note Preemption must be disabled to use this.
  */
 
-#ifdef CONFIG_PREEMPT_RT
-static DEFINE_RAW_SPINLOCK(_stp_print_lock);
-#else
 static DEFINE_SPINLOCK(_stp_print_lock);
-#endif
 
 void EXPORT_FN(stp_print_flush)(_stp_pbuf *pb)
 {
@@ -99,11 +95,13 @@ void EXPORT_FN(stp_print_flush)(_stp_pbuf *pb)
 #else  /* !STP_BULKMODE */
 
 #if STP_TRANSPORT_VERSION == 1
-	/** STP_TRANSPORT_VERSION == 1 is special, _stp_ctl_write will
+	/** STP_TRANSPORT_VERSION == 1 is special, _stp_ctl_send will
 	    pass through procfs _stp_ctl_write_fs which recognizes
-	    STP_REALTIME_DATA as data that needs to be send right away
-	    over the .cmd channel instead of being queued.  */
-	if (unlikely(_stp_ctl_write(STP_REALTIME_DATA, pb->buf, len) <= 0))
+	    STP_REALTIME_DATA as data that can be concatenated if the
+	    previous buffer is also of type STP_REALTIME_DATA and there
+	    is some room left in that packet instead of creating a new
+	    packet to be queued.  */
+	if (unlikely(_stp_ctl_send(STP_REALTIME_DATA, pb->buf, len) <= 0))
 		atomic_inc (&_stp_transport_failures);
 
 #else  /* STP_TRANSPORT_VERSION != 1 */
