@@ -11,8 +11,6 @@
 /* amount of data a print can send. */
 #define STP_BUFFER_SIZE 8192
 
-static int _stp_ctl_write(int type, void *data, unsigned len);
-
 /* STP_CTL_BUFFER_SIZE is the maximum size of a message */
 /* exchanged on the control channel. */
 #if STP_TRANSPORT_VERSION == 1
@@ -22,11 +20,9 @@ static int _stp_ctl_write(int type, void *data, unsigned len);
 #define STP_CTL_BUFFER_SIZE 256
 #endif
 
-/* how often the work queue wakes up and checks buffers */
-#define STP_WORK_TIMER (HZ/100)
-
 static unsigned _stp_nsubbufs;
 static unsigned _stp_subbuf_size;
+static pid_t _stp_target;
 
 static int _stp_transport_init(void);
 static void _stp_transport_close(void);
