@@ -115,7 +115,7 @@ int insert_module(
 	   point on to avoid TOCTOU problems. */
 	module_fd = open(module_realpath, O_RDONLY);
 	if (module_fd < 0) {
-		perr("Error opening '%s'", module_realpath);
+		perr("Couldn't open '%s'", module_realpath);
 		free(opts);
 		return -1;
 	}
@@ -489,7 +489,7 @@ check_stap_module_path(const char *module_path, int module_fd)
 		 * checked the length of the resulting string.  */
 		strcat(staplib_dir_realpath, "/");
 	else {
-		err("ERROR: Path \"%s\" is too long.", staplib_dir_realpath);
+		err("Path \"%s\" is too long.", staplib_dir_realpath);
 		return -1;
 	}
 
@@ -505,7 +505,7 @@ check_stap_module_path(const char *module_path, int module_fd)
 	}
 	/* Make sure it is a directory. */
 	if (! S_ISDIR(sb.st_mode)) {
-		err("ERROR: Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
+		err("Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
 		    "  the \"%s\" directory.\n"
 		    "  That path must refer to a directory.\n",
 		    staplib_dir_path);
@@ -513,7 +513,7 @@ check_stap_module_path(const char *module_path, int module_fd)
 	}
 	/* Make sure it is owned by root. */
 	if (sb.st_uid != 0) {
-		err("ERROR: Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
+		err("Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
 		    "  the \"%s\" directory.\n"
 		    "  That directory should be owned by root.\n",
 		    staplib_dir_path);
@@ -521,7 +521,7 @@ check_stap_module_path(const char *module_path, int module_fd)
 	}
 	/* Make sure it isn't world writable. */
 	if (sb.st_mode & S_IWOTH) {
-		err("ERROR: Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
+		err("Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
 		    "  the \"%s\" directory.\n"
 		    "  That directory should not be world writable.\n",
 		    staplib_dir_path);
@@ -532,7 +532,7 @@ check_stap_module_path(const char *module_path, int module_fd)
 	 * module_path starts with staplib_dir_realpath. */
 	if (strncmp(staplib_dir_realpath, module_path,
 		    strlen(staplib_dir_realpath)) != 0) {
-		err("ERROR: Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
+		err("Members of the \"stapusr\" and \"stapsys\" groups can only use unsigned modules within\n"
 		    "  the \"%s\" directory.\n"
 		    "  Module \"%s\" does not exist within that directory.\n",
 		    staplib_dir_path, module_path);
@@ -541,17 +541,17 @@ check_stap_module_path(const char *module_path, int module_fd)
 
 	/* Validate the module permisions. */
 	if (fstat(module_fd, &sb) < 0) {
-		perr("Error getting information on the module\"%s\"", module_path);
+		perr("Couldn't get information on the module\"%s\"", module_path);
 		return -1;
 	}
 	/* Make sure it is owned by root. */
 	if (sb.st_uid != 0) {
-		err("ERROR: The module \"%s\" must be owned by root.\n", module_path);
+		err("The module \"%s\" must be owned by root.\n", module_path);
 		rc = -1;
 	}
 	/* Make sure it isn't world writable. */
 	if (sb.st_mode & S_IWOTH) {
-		err("ERROR: The module \"%s\" must not be world writable.\n", module_path);
+		err("The module \"%s\" must not be world writable.\n", module_path);
 		rc = -1;
 	}
 
@@ -598,7 +598,7 @@ static privilege_t get_module_required_credentials (
      actual privilege level.
   */
   if (verbose >= 1) {
-    err ("Unable to determine the privilege level required for the module %s. Assuming %s.\n",
+    eprintf ("Unable to determine the privilege level required for the module %s. Assuming %s.\n",
 	 module_path, pr_name (pr_stapusr));
   }
   return pr_stapusr;
@@ -617,9 +617,9 @@ static privilege_t get_module_required_credentials (
   scn = find_section_in_module (module_file, st_size, STAP_PRIVILEGE_SECTION);
   if (! scn) {
     if (verbose >= 1) {
-      err ("Section name \"%s\" not found in module %s.\n", STAP_PRIVILEGE_SECTION,
+      eprintf ("Section name \"%s\" not found in module %s.\n", STAP_PRIVILEGE_SECTION,
 	   module_path);
-      err ("Assuming required privilege level of %s.\n", pr_name (pr_stapusr));
+      eprintf ("Assuming required privilege level of %s.\n", pr_name (pr_stapusr));
     }
     return pr_stapusr;
   }
@@ -630,9 +630,9 @@ static privilege_t get_module_required_credentials (
      Get the section header. */
   if (gelf_getshdr (scn, & shdr) == NULL) {
     if (verbose >= 1) {
-      err ("Error getting section header from section %s in module %s.\n", STAP_PRIVILEGE_SECTION,
+      eprintf ("Error getting section header from section %s in module %s.\n", STAP_PRIVILEGE_SECTION,
 	   module_path);
-      err ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
+      eprintf ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
     }
     return pr_highest;
   }
@@ -640,9 +640,9 @@ static privilege_t get_module_required_credentials (
   /* The section should have at least one data item. */
   if (shdr.sh_size < 1) {
     if (verbose >= 1) {
-      err ("Section header from section %s in module %s has no items\n", STAP_PRIVILEGE_SECTION,
+      eprintf ("Section header from section %s in module %s has no items\n", STAP_PRIVILEGE_SECTION,
 	   module_path);
-      err ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
+      eprintf ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
     }
     return pr_highest;
   }
@@ -650,9 +650,9 @@ static privilege_t get_module_required_credentials (
   /* The first data item contains the privilege requirement of the module. */
   if ((data = elf_getdata (scn, data)) == NULL) {
     if (verbose >= 1) {
-      err ("Error getting data from section %s in module %s\n", STAP_PRIVILEGE_SECTION,
+      eprintf ("Error getting data from section %s in module %s\n", STAP_PRIVILEGE_SECTION,
 	   module_path);
-      err ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
+      eprintf ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
     }
     return pr_highest;
   }
@@ -660,9 +660,9 @@ static privilege_t get_module_required_credentials (
   /* Make sure the data is the correct size. */
   if (data->d_size != sizeof (privilege)) {
     if (verbose >= 1) {
-      err ("Data in section %s is in module %s not the correct size\n", STAP_PRIVILEGE_SECTION,
+      eprintf ("Data in section %s is in module %s not the correct size\n", STAP_PRIVILEGE_SECTION,
 	   module_path);
-      err ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
+      eprintf ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
     }
     return pr_highest;
   }
@@ -678,7 +678,7 @@ static privilege_t get_module_required_credentials (
     break; /* ok */
   default:
     if (verbose >= 1) {
-      err ("Unknown privilege data, 0x%x in section %s in module %s\n",
+      eprintf ("Unknown privilege data, 0x%x in section %s in module %s\n",
 	   (int)privilege, STAP_PRIVILEGE_SECTION, module_path);
       err ("Assuming required privilege level of %s.\n", pr_name (pr_highest));
     }
@@ -735,7 +735,7 @@ check_groups (
     }
 
     /* Our credentials are insufficient to load this module. */
-    err("ERROR: Your privilege credentials (%s) are insufficient to load the module %s (%s required)\n",
+    err("Your privilege credentials (%s) are insufficient to load the module %s (%s required)\n",
 	pr_name (user_credentials), module_path, pr_name (module_required_credentials));
 
     if (user_credentials == pr_none)
@@ -796,13 +796,13 @@ void assert_stap_module_permissions(
 		/* ... like overriding the real UID */
 		const char *env_id = getenv("SYSTEMTAP_REAL_UID");
 		if (env_id && setreuid(atoi(env_id), -1))
-			err("WARNING: couldn't set staprun UID to '%s': %s",
+			warn("Couldn't set staprun UID to '%s': %s",
 					env_id, strerror(errno));
 
 		/* ... or overriding the real GID */
 		env_id = getenv("SYSTEMTAP_REAL_GID");
 		if (env_id && setregid(atoi(env_id), -1))
-			err("WARNING: couldn't set staprun GID to '%s': %s",
+			warn("Couldn't set staprun GID to '%s': %s",
 					env_id, strerror(errno));
 
 		if (user_credentials)
@@ -819,7 +819,7 @@ void assert_stap_module_permissions(
 
 	/* Are we are an ordinary user?.  */
 	if (check_groups_rc == -2) {
-		err("ERROR: You are trying to run systemtap as a normal user.\n"
+		err("You are trying to run systemtap as a normal user.\n"
 		    "You should either be root, or be part of "
 		    "group \"stapusr\" and possibly the groups \"stapsys\" or \"stapdev\".\n");
 	}
@@ -871,7 +871,7 @@ void assert_uprobes_module_permissions(
 
 	/* Check permissions for group membership.  */
 	if (check_groups_rc == -2) {
-		err("ERROR: You are trying to load the module %s as a normal user.\n"
+		err("You are trying to load the module %s as a normal user.\n"
 		    "You should either be root, or be part of "
 		    "group \"stapusr\" and possibly the groups \"stapsys\" or \"stapdev\".\n",
 		    module_path);
