@@ -281,19 +281,22 @@ void target_symbol::chain (const semantic_error &er)
   this->saved_conversion_error = e;
 }
 
+
 string target_symbol::sym_name ()
 {
-  if (name == "@var")
-    {
-      if (cu_name == "")
-	return target_name;
-      else
-	return target_name.substr(0, target_name.length() - cu_name.length() - 1);
-    }
-  else
-    return name.substr(1);
+  return name.substr(1);
 }
 
+
+string atvar_op::sym_name ()
+{
+  if (cu_name == "")
+    return target_name;
+  else
+    return target_name.substr(0, target_name.length() - cu_name.length() - 1);
+}
+
+
 // ------------------------------------------------------------------------
 // parse tree printing
 
@@ -345,7 +348,7 @@ void regex_query::print (ostream& o) const
   // a parenthesized RHS.
   o << "(" << *left << ") "
     << op
-    << " " << *re;
+    << " " << *right;
 }
 
 
@@ -416,8 +419,16 @@ void target_symbol::print (ostream& o) const
   if (addressof)
     o << "&";
   o << name;
-  if (name == "@var")
-    o << "(\"" << target_name << "\")";
+  for (unsigned i = 0; i < components.size(); ++i)
+    o << components[i];
+}
+
+
+void atvar_op::print (ostream& o) const
+{
+  if (addressof)
+    o << "&";
+  o << name << "(\"" << target_name << "\")";
   for (unsigned i = 0; i < components.size(); ++i)
     o << components[i];
 }
@@ -1494,6 +1505,13 @@ cast_op::visit (visitor* u)
 }
 
 
+void
+atvar_op::visit (visitor* u)
+{
+  u->visit_atvar_op(this);
+}
+
+
 void
 defined_op::visit (visitor* u)
 {
@@ -1780,7 +1798,7 @@ void
 traversing_visitor::visit_regex_query (regex_query* e)
 {
   e->left->visit (this);
-  e->right->visit (this); // TODOXXX do we need to traverse the literal in RHS?
+  e->right->visit (this);
 }
 
 void
@@ -1830,6 +1848,12 @@ traversing_visitor::visit_cast_op (cast_op* e)
   e->visit_components (this);
 }
 
+void
+traversing_visitor::visit_atvar_op (atvar_op* e)
+{
+  e->visit_components (this);
+}
+
 void
 traversing_visitor::visit_defined_op (defined_op* e)
 {
@@ -2016,6 +2040,19 @@ varuse_collecting_visitor::visit_target_symbol (target_symbol *e)
   functioncall_traversing_visitor::visit_target_symbol (e);
 }
 
+
+void
+varuse_collecting_visitor::visit_atvar_op (atvar_op *e)
+{
+  // Similar to visit_target_symbol
+
+  if (is_active_lvalue (e))
+    embedded_seen = true;
+
+  functioncall_traversing_visitor::visit_atvar_op (e);
+}
+
+
 void
 varuse_collecting_visitor::visit_cast_op (cast_op *e)
 {
@@ -2461,6 +2498,12 @@ throwing_visitor::visit_target_symbol (target_symbol* e)
   throwone (e->tok);
 }
 
+void
+throwing_visitor::visit_atvar_op (atvar_op* e)
+{
+  throwone (e->tok);
+}
+
 void
 throwing_visitor::visit_cast_op (cast_op* e)
 {
@@ -2753,6 +2796,13 @@ update_visitor::visit_cast_op (cast_op* e)
   provide (e);
 }
 
+void
+update_visitor::visit_atvar_op (atvar_op* e)
+{
+  e->visit_components (this);
+  provide (e);
+}
+
 void
 update_visitor::visit_defined_op (defined_op* e)
 {
@@ -3009,6 +3059,12 @@ deep_copy_visitor::visit_cast_op (cast_op* e)
   update_visitor::visit_cast_op(new cast_op(*e));
 }
 
+void
+deep_copy_visitor::visit_atvar_op (atvar_op* e)
+{
+  update_visitor::visit_atvar_op(new atvar_op(*e));
+}
+
 void
 deep_copy_visitor::visit_defined_op (defined_op* e)
 {
