@@ -278,7 +278,7 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       s.op->newline(1) << "struct sk_buff *nf_skb = nf_pskb ? *nf_pskb : NULL;";
 
       s.op->newline(-1) << "#endif";
-      s.op->newline(1) << "struct stap_probe * const stp = & stap_probes[" << np->session_index << "];";
+      s.op->newline(1) << "const struct stap_probe * const stp = & stap_probes[" << np->session_index << "];";
       s.op->newline() << "int nf_verdict = NF_ACCEPT;"; // default NF_ACCEPT, to be used by $verdict context var
       common_probe_entryfn_prologue (s, "STAP_SESSION_RUNNING", "stp",
                                      "stp_probe_type_netfilter",
@@ -314,7 +314,7 @@ netfilter_derived_probe_group::emit_module_decls (systemtap_session& s)
       // Invoke the probe handler
       s.op->newline() << "(*stp->ph) (c);";
 
-      common_probe_entryfn_epilogue (s.op, false, s.suppress_handler_errors);
+      common_probe_entryfn_epilogue (s, false);
 
       if (np->context_vars.find("__nf_verdict") != np->context_vars.end())
         s.op->newline() << "nf_verdict = (int) "+c_p+"." + s.up->c_localname("__nf_verdict") + ";";
